package MyVector;

my $pkg="MyVector";

use strict;
use lib ($ENV{MYCONF_LIB});
use My;
use Write;
use Class::Date qw(date gmdate);
use Image::ExifTool; # baliky kolem Exif: libimage-exif-perl, libimage-exiftool-perl, libimage-metadata-jpeg-perl

#== Napojeni na databazi
my $dbh=My->dbh;


# zjisteni (nebo vytvoreni) trackid
sub trackidbyremoteuser {
   my ($pkg_p, $tracknamewish)=@_;
   my $trackid;
   my $trackname;
   if(length($tracknamewish)) {
      $trackname=$tracknamewish;
   }
   else {
      $trackname=My->remote_user;
   }
   my $sthti=$dbh->prepare(q{
      select
         Trackinfo.trackid
      from
         MyVector.Trackinfo
      where
         Trackinfo.trackname=?
   });
   $sthti->execute($trackname);
   if(my $refti=$sthti->fetchrow_hashref) {
      $trackid=$$refti{trackid};
   }
   else {
      Write->insert({
         _table => "MyVector.Trackinfo",
         trackname => $trackname,
         userwrite => My->remote_user,
         datewrite => My->date_now,
         _bezdatzapis => 1
      });
      # zjisteni vlozeneho trackid
      my $sth=$dbh->prepare(q{
         select
            last_insert_id() as trackid
      });
      $sth->execute;
      my $ref=$sth->fetchrow_hashref;
      $trackid=$$ref{trackid};
      $sth->finish;
   }
   $sthti->finish;
   return $trackid;
}

sub recalclocation {
   my ($pkg_p, $ref)=@_;
   my $sth=$dbh->prepare(q{
      select
         Track.*,
         abs(timestampdiff(SECOND,Track.datesat,?)) as secdiff
      from
         MyVector.Track
      where
         Track.trackid=? and
         Track.datesat>=? and
         Track.datesat<=?
      order by
         secdiff asc
      limit
         1
   });
   my $maxsecdiff=3*60; # 3 minuty
   my $mindatesatD=gmdate($$ref{datesat})-($maxsecdiff.'s');
   my $maxdatesatD=gmdate($$ref{datesat})+($maxsecdiff.'s');
   $sth->execute(
      $$ref{datesat},
      $$ref{trackid},
      $mindatesatD->strftime("%Y-%m-%d %X"),
      $maxdatesatD->strftime("%Y-%m-%d %X")
   );
   if(my $refnew=$sth->fetchrow_hashref) {
      $$ref{Pla}=$$refnew{Pla};
      $$ref{Plo}=$$refnew{Plo};
      $$ref{PA}=$$refnew{PA};
      my $sign;
      if($maxsecdiff le $$refnew{datesat}) {
         $sign='+';
      }
      else {
         $sign='-';
      }
      $$ref{provider}="time".$sign.$$refnew{secdiff}."s";
   }
   $sth->finish;
}

sub writeshare {
   my ($pkg_p, $trackid,$img,$corectiontogmt)=@_;
   my $statustext;
   my $datesat=date(time)->to_tz("GMT")->strftime("%Y-%m-%d %X");
   my $exifTool = new Image::ExifTool;
   if($exifTool->ExtractInfo(\$img)) {
      my $gpsinfo="bad";
      my $write={
         _table => "MyVector.Share",
         body => $img,
         datesat => $datesat,
         Pla => 91, # preddefinovana je schvalne nesmyslna hodnota
         Plo => 0,
         PA => 0,
         azimuth => 0,
         accuracy => 0,
         provider => "?",
         userwrite => My->remote_user,
         datewrite => My->datetime_now,
         _bezdatzapis => 1
      };
      my @tagList = qw(CreateDate DateTimeOriginal GPSDateStamp GPSTimeStamp GPSAltitude GPSAltitudeRef GPSLatitude GPSLatitudeRef GPSLongitude GPSLongitudeRef);
      foreach my $tag (@tagList) {
         my $val = $exifTool->GetValue($tag, 'ValueConv');
         if(ref($val) ne 'ARRAY' and ref($val) ne 'SCALAR') {
            if(length($val)) {
               if($tag eq "CreateDate" or $tag eq "DateTimeOriginal") {
                  my $datetimestr=$val;
                  if($datetimestr=~/^(\d{4})\:(\d{2})\:(\d{2}) (\d{2}\:\d{2}\:\d{2})$/) {
                     $datetimestr=$1."-".$2."-".$3." ".$4;
                     if(length($corectiontogmt)) {
                        my $datesatD=gmdate($datetimestr)-($corectiontogmt.'s');
                        $$write{datesat}=$datesatD->strftime("%Y-%m-%d %X");
                     }
                     else {
                        # mlcky predpokladam, ze na zarizeni je stejna casova zone jako na serveru
                        # - coz nemusi byt pravda - asi bych mel posilat i casovou zonu na zarizeni
                        #   (kazdopadne nejlepsi je pokud je v obrazku ulozen i
                        #   GPS cas - pak neni problem)
                        $$write{datesat}=date($datetimestr)->to_tz("GMT")->strftime("%Y-%m-%d %X");
                     }
                  }
               }
               elsif($tag eq "GPSDateStamp") {
                  my $datestr=$val;
                  if($datestr=~/^(\d{4})\:(\d{2})\:(\d{2})$/) {
                     $datestr=$1."-".$2."-".$3;
                     $$write{datesat}=$datestr." ".substr($$write{datesat},11);
                  }
               }
               elsif($tag eq "GPSTimeStamp") {
                  my $timestr=$val;
                  if($timestr=~/^(\d{2})\:(\d{2})\:(\d{2})$/) {
                     $timestr=$1.":".$2.":".$3;
                     $$write{datesat}=substr($$write{datesat},0,10)." ".$timestr;
                  }
               }
               elsif($tag eq "GPSAltitude") {
                  $$write{PA}=$val;
               }
               elsif($tag eq "GPSLatitude") {
                  $$write{Pla}=$val;
                  $$write{provider}="gps";
                  $gpsinfo="ok";
               }
               elsif($tag eq "GPSLatitudeRef") {
                  if($val eq "S") {
                     $$write{Pla}*=-1;
                  }
               }
               elsif($tag eq "GPSLongitude") {
                  $$write{Plo}=$val;
               }
               elsif($tag eq "GPSLongitudeRef") {
                  if($val eq "W") {
                     $$write{Plo}*=-1;
                  }
               }
            }
         }
      }
      # DUPLICITA?
      # jde o jiz jednou zaslany snimek? - duplicity poznavam zatim na
      # zaklade casu - nepredpokladam, ze jedina osoba udela vice snimku
      # behem 1s
      # bohuzel GPS cas porizeny originalni aplikaci fotoaparatu se pri
      # rychlem foceni (urcite behem minuty) za sebou nemeni a nemeni
      # se ani souradnice (aplikace si proste zjisti souradnici a s
      # tou je asi nejakou dobu - mozna cele foceni do jejiho zavreni)
      # spokojena - proto jeste delam kontrolu na velikost snimku
      my $shareid;
      my $sth=$dbh->prepare(q{
         select
            Share.shareid
         from
            MyVector.Share
         where
            Share.trackid=? and
            Share.datesat=? and
            length(Share.body)=?
         limit
            1
      });
      $sth->execute($trackid,$$write{datesat},length($$write{body}));
      if(my $ref=$sth->fetchrow_hashref) {
         $shareid=$$ref{shareid};
      }
      $sth->finish;
      # zatim 
      if(defined($shareid)) {
         $$write{_key_trackid}=$trackid,
         $$write{_key_shareid}=$shareid,
         Write->replace($write);
         $statustext="replace photo" . " (GPS info " . $gpsinfo . ")";
      }
      else {
         $$write{trackid}=$trackid,
         Write->insert($write);
         $statustext="insert photo" . " (GPS info " . $gpsinfo . ")";
      }
   }
   return $statustext;
}

# vraci adresu obrazku se srazkami dle zadaneho $datesat
# - pokud jeste neni obrazek na serveru, tak se jej pokusi natahnout
# - vrati undef pokud obrazek neexistuje (a ani se jej nezdarilo natahnout)
sub rainfallImgsrc {
   my ($pkg_p, $datesat)=@_;
   my $localpath='overlay/rainfall/';
   my $proj="M"; # pokud pouzijej my $proj="", nebudes provadet konverzi projekce a nejak to tam napasujes; u my $proj="M" provadis konverzi projekce na mercator
   my $datetimegmt_now=gmdate(time-75)->strftime("%Y-%m-%d %X"); # umyslne posouvam aktualni cas o 75s zpet, protoze radarove snimky jsou zverejnovany asi s minutovym zpozdeni od casu, ktereho se tykaji (delam to proto at se zbytecne nesnazim celou minutu ziskavat obrazek, ktery tam jeste temer jiste neni)
   if($datesat ge $datetimegmt_now) {
      $datesat=$datetimegmt_now;
   }
   my $gmdateD=gmdate(gmdate($datesat)->strftime("%Y-%m-%d %H:%M:00"));
   my $delta;
   my $minutes=$gmdateD->strftime("%M");
   if($minutes>=45) {
      $delta=$minutes-45;
   }
   elsif($minutes>=30) {
      $delta=$minutes-30;
   }
   elsif($minutes>=15) {
      $delta=$minutes-15;
   }
   else {
      $delta=$minutes-0;
   }
   my $gmdateloadD=$gmdateD-($delta.'min');
   my $tmpout="/tmp/chmi_".$$.".png";
   # Poznamka:
   # - na portal.chmi.cz zel jsou data jen nekolik dni dozadu
   # - zjistil jsem, ze se da trochu krkolomnym zpusobem natahnout
   #   i mnohem starsi data z idnes - priklad:
   #   - wget "http://pocasi.idnes.cz/utf/normal@meteodata.aspx?t=image&d=aladin_srazky&c=10.1.2011%2013:00:00" -O out.txt
   #   - less out.txt
   #   - => http://i.idnes.cz/pocasi/11/01/radar/1136227_pacz23.z_max3d.20110110.1200.0_n.jpg
   #   -    testovanim jsem pak prisel na original soubor
   #        => http://i.idnes.cz/pocasi/11/01/radar/1136227_pacz23.z_max3d.20110110.1200.0.png
   my $urlbase="http://portal.chmi.cz/files/portal/docs/meteo/rad/data_tr_png_1km/pacz23.z_max3d.";
   for(my $i=1;$i<=2;++$i) {
      # obrazek neni natazen a upraven z divejska
      if(!-e($localpath.$gmdateloadD->strftime("%Y%m%d.%H%M".$proj.".png"))) {
         # natazeni obrazku
         my $url=$urlbase . $gmdateloadD->strftime("%Y%m%d.%H%M.0.png");
         system('LANG=C wget -q -O "' . $tmpout . '" "' . $url . '" --dns-timeout=2 --connect-timeout=3 --read-timeout=6 2> /dev/null');
         if(!-z($tmpout)) {
            last;
         }
         else {
            $gmdateloadD=$gmdateloadD-('15min');
         }
      }
      # obrazek je jiz natazen a upraven z divejska
      else {
         $tmpout=undef;
         last;
      }
   }
   # uprava natazeneho obrazku a umisteni na finalni misto
   if(defined($tmpout) and !-z($tmpout)) {
      require Image::Magick;
      my $image = Image::Magick->new;
      $image->Read($tmpout);
      $image->Crop(geometry=>'728x528+0+82');
      $image->Transparent(color=>'#c4c4c4ff');
      $image->Transparent(color=>'#000000ff');
      $image->Write($localpath.$gmdateloadD->strftime("%Y%m%d.%H%M.png"));
      unlink($tmpout);
      if($proj eq "M") {
         my $tmpouttiff="/tmp/chmi_".$$.".tiff";
         my $tmpouttiffM="/tmp/chmi_".$$."M.tiff";
         # mam to postaveno na urceni GPS souradnic k nekolika bodum mapy (nevyuzivam definice projekce obrazku, ktera pravdepodobne je: -a_srs '+proj=gnom +lat_0=50.008 +lon_0=14.447 +x_0=False Easting +y_0=False Northing'
         system(
            'gdal_translate -of GTiff' .
            ' -gcp 293 184 14.30959 51.05896 -gcp 622 344 18.84719 49.51774 -gcp 485 450 16.93914 48.61779 -gcp 256 438 13.84023 48.77107 -gcp 134 262 12.10186 50.31982 -gcp 535 260 17.69825 50.32291' .
            ' "' . $localpath.$gmdateloadD->strftime("%Y%m%d.%H%M.png") . '" "' . $tmpouttiff . '" > /dev/null'
         );
         system(
            'gdalwarp -t_srs "+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +wktext +no_defs"' .
            ' "' . $tmpouttiff . '" "' . $tmpouttiffM . '" > /dev/null'
         );
         my $image = Image::Magick->new;
         $image->Read($tmpouttiffM);
         $image->Write($localpath.$gmdateloadD->strftime("%Y%m%d.%H%M".$proj.".png"));
         unlink($tmpouttiff);
         unlink($tmpouttiffM);
      }
   }
   my $imgSrc;
   if(-e($localpath.$gmdateloadD->strftime("%Y%m%d.%H%M".$proj.".png"))) {
      $imgSrc=$localpath.$gmdateloadD->strftime("%Y%m%d.%H%M".$proj.".png");
   }
   return $imgSrc;
}

return 1;

